
import java.awt.*;
import java.awt.geom.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.*;

/**
 * Clase de dibujo del piso.
 *
 * @author Agustin Gonzales, Victor Arredondo, Yen-kung Yu
 */
class GPiso extends Piso implements Drawable {

    private float width;
    private BufferedImage img = null;
    private Color color = Color.GREEN;
    private Rectangle2D.Float shape = null;

    /**
     * Constructor GPiso Inicializa un nuevo GPiso.
     *
     * @param pos Posicion inicial del objeto creado.
     * @param w Ancho y largo de cada elemento de piso.
     */
    public GPiso(Vector2D pos, float w) {
        super(pos);
        width = w;
        readImage();
        shape = new Rectangle2D.Float((float) pos.getX() - w / 2, (float) pos.getY() - w / 2, w, w);
    }

    /**
     * Dibuja un elemento de piso.
     *
     * @param g Objeto grafico de 2 dimensiones.
     */
    public void draw(Graphics2D g) {
        shape.setFrame((float) getX() - width / 2, (float) getY() - width / 2, width, width);
        g.fill(shape);
        g.drawImage(img, (int) (getX() - width / 2), (int) (getY() - width / 2), (int) width, (int) width, null);
    }

    /**
     * Lee un archivo imagen en el disco duro.
     */
    public void readImage() {

        try {
            img = ImageIO.read(new File("piso3.jpg"));
        } catch (IOException ex) {
            System.out.println("No se pudo leer la imagen");
        }
    }

    /**
     * Determina si se contiene un objeto.
     *
     * @param x Coordenada X de la posicion.
     * @param y Coordenada Y de la posicion.
     * @return True si contiene a otro, False si no.
     */
    public boolean contains(int x, int y) {
        return shape.contains(x, y);
    }

    /**
     * Establece la posicion del bloque.
     *
     * @param x Coordenada X de la posicion.
     * @param y Coordenada Y de la posicion.
     */
    public void dragTo(int x, int y) {
    }

    /**
     * Establece la posicion a mover.
     */
    public void moveTo(double x, double y) {
        setPosition(x, y);
    }

    /**
     * Indica que el bloque se ha seleccionado.
     */
    public void setSelected() {
        color = Color.RED;
    }

    /**
     * Se deja de marcar el bloque como seleccionado.
     */
    public void setReleased() {
        color = Color.GREEN;
    }
}
