
import javax.swing.Timer;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Graphics2D;
import java.util.*;
import java.awt.Cursor;
import java.applet.AudioClip;
import java.net.URL;
import java.io.*;
import javax.sound.sampled.*;
/**
 * Clase encargada de la simulación del sismo. Realiza la interacción entre los distintos objetos en el panel.
 * @author Agustin G.,Victor,Yen
 */
public class MyWorld implements ActionListener {

    public static int WIDTH = 1500;  // in pixels
    public static int HEIGHT = 700; // in pixels
    private double t;              // Simulation time
    private Timer pase;
    private double delta_t;        // in seconds
    private double refreshPeriod;  // in seconds
    private Vector2D g;
    private float viscosity;
    private PhysicsLabPanel labPanel;
    private Vector<PhysicsElement> elements;
    private int cont;
    private int count2;
    private int cont3;
    public static AudioClip earth;
    private double trans;
    public double intensity;
    private double k;
/** 
 * Constructor MyWorld.
 */
    public MyWorld() {
        cont3 = 0;
        intensity = 1;
        URL sound = this.getClass().getClassLoader().getResource("earth.wav");
        earth = java.applet.Applet.newAudioClip(sound);
        cont = 0;
        count2 = 1;
        t = 0;
        refreshPeriod = 0.06;      // 80 [ms]
        delta_t = 0.01;          // 0.1 [ms]
        pase = new Timer((int) (refreshPeriod * 1000), this);
        g = new Vector2D(0, 0);
        viscosity = 0.0f;
        elements = new Vector<PhysicsElement>();
        labPanel = null;
    }
/** 
 *  Fija el panel al objeto MyWorld
 * @param  p 	Panel 
 */ 
    public void setPanel(PhysicsLabPanel p) {
        labPanel = p;
    }
/** 
 *  Fija el panel periodo de muestreo de la simulación
 * @param delta tiempo de muestreo
 */ 
    public void setDelta_t(double delta) {
        delta_t = delta;
    }
/** 
 *  Fija el panel periodo en que actualiza los elementos en el panel
 * @param rp tiempo de muestreo
 */ 
    public void setRefreshPeriod(double rp) {
        refreshPeriod = rp;
        pase.setDelay((int) (refreshPeriod * 1000));
    }

    public void setIntensity(double inten) {
        intensity = inten;
    }

    public void setGravity(Vector2D gravity) {
        g = gravity;
    }

    public void setViscosity(float v) {
        viscosity = v;
    }

    public void addElement(PhysicsElement e) {
        elements.add(e);
        labPanel.repaint();
    }

    public void removeElement(PhysicsElement e) {
        if (e == null) {
            return;
        }
        elements.remove(e);
        labPanel.repaint();
    }

    public void start() { //Este método debe ser modificado
        Vector2D zero = new Vector2D(); //Se inicializa en (0,0)
	setRefreshPeriod(0.001);
        setViscosity(0.1f);
        setGravity(new Vector2D(9.8,0));
        if (pase.isRunning()) {
            return;
        }
        for (PhysicsElement e : elements) {
            e.setInitialState(g, viscosity);
            if (e instanceof Bloque) {// Si el elemento es una Bloque, entonces se deben resetear la velocidad y la aceleración
                ((Bloque) e).setSpeed(zero);//Se definio el metodo en la clase Bloque
                ((Bloque) e).setAccel(zero);//Idem a setSpeed
            }
        }

        labPanel.desableMouseListener();
        pase.start();

    }

    public void resume() {// El método start ahora es Resume
        earth.play();

        if (pase.isRunning()) {
            return;
        }
        for (PhysicsElement e : elements) {
            e.setInitialState(g, viscosity);
        }
        labPanel.desableMouseListener();
        pase.start();

    }

    public void stop() {
        earth.stop();
        pase.stop();
        //labPanel.message();
        labPanel.enableMouseListener();
    }
/** 
 *  Realiza El movimiento sísmico del entorno gráfico y realiza la operación de asignarle velocidades a los objetos Bloque
 * @param  e 	Elemento físico 
 */ 
    public void shake(PhysicsElement e) {

        if (e instanceof GPiso) {
            if (cont == 0) {
                cont = 30;
                trans = -k;
                if (count2 == 2) {
                    k = (Math.random() * ((30) + 1) - 20);
                    trans = k;
                    count2 = 0;
                }
                count2++;
            }
            cont--;
            ((Piso) e).setPosition(((Piso) e).getX() + trans * intensity, ((Piso) e).getY());

        }
        if (e instanceof GBloque) {
            ((Bloque) e).setSpeed(((Bloque) e).getSpeed().plus(new Vector2D(-trans * intensity * 8, 0)));
        }

    }
/** 
 *  Simulación del sismo 
 * @param  event evento al presionar start 
 */ 
    public void actionPerformed(ActionEvent event) { 
        if (cont3 == 0) {
            earth.play();
            cont3 = 1200;
        }
        cont3--;
        double nextStop = t + refreshPeriod;
        trans = 0;
        for (; t < nextStop; t += delta_t) {
            for (PhysicsElement e : elements) {

                shake(e);
                e.computeNextState(delta_t, g, viscosity);
            }
            for (PhysicsElement e : elements) {
                e.updateState();
            }
        }
        labPanel.repaint();
    }

    public void paintComponent(Graphics2D g2d) {
        for (PhysicsElement e : elements) {
            if (e instanceof Drawable) {
                ((Drawable) e).draw(g2d);
            }
        }
    }

    public Drawable find(int x, int y) {
        for (PhysicsElement e : elements) {
            if (e instanceof Drawable) {
                Drawable de = (Drawable) e;
                if (de.contains(x, y)) {
                    return de;
                }
            }
        }
        return null;
    }

    public void setCursor() {
        labPanel.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
    }

    public void resetCursor() {
        labPanel.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
/** 
 *  Busca un elemento que herede de la clase AttachableElement y lo retorna si es que lo encuentra.
 * @param  x,y posicion del objeto del tipo AttachableElement
 */ 
    public AttachableElement findAttachableElement(int x, int y) {
        for (PhysicsElement e : elements) {
            if (e instanceof AttachableElement) {
                AttachableElement ae = (AttachableElement) e;
                if (ae instanceof Drawable) {
                    if (((Drawable) ae).contains(x, y)) {
                        return ae;
                    }
                }
            }
        }
        return null;
    }
}
