public class Pilar extends PhysicsElement  {
   private static int id=0;  // Pilar identification
   protected final float largonat;
   private final float rigidez;
   protected AttachableElement a_end, b_end;

   private Pilar(){   // nobody can create a block without state
      super(id++);
      rigidez= largonat = 0;
   }
   public Pilar(float largonat, float rigidez){
      super(id++);
      this.largonat = largonat;
      this.rigidez= rigidez;
      a_end = b_end = null;
   }
   public void attachAend (AttachableElement ae) {
      a_end = ae;
      ae.attachPilar(this);
   }
   public void detachAend(){
      if (a_end == null) return;
      a_end.detachPilar(this);
      a_end=null;
   }
   
   public void attachBend (AttachableElement ae) {
      b_end = ae;
      ae.attachPilar(this);
   }
   public void detachBend(){
     if (b_end == null) return;
     b_end.detachPilar(this);
     b_end = null;
   }
   public Vector2D getAendPosition() {
      if (a_end != null)
         return a_end.getPosition();
      if (b_end != null)
         return b_end.getPosition().minus(new Vector2D(0,largonat));
      return new Vector2D();
   }
   public Vector2D getBendPosition() {
      if (b_end != null)
         return b_end.getPosition();
      if (a_end != null)
         return a_end.getPosition().plus(new Vector2D(0,largonat));
      return new Vector2D(0,largonat);
   }
   protected Vector2D getVector() {
      return getBendPosition().minus(getAendPosition());
   }
   
   public Vector2D getForce(AttachableElement ae) {
      Vector2D force = new Vector2D();
      if ((a_end == null) || (b_end == null))
         return force;
      if ((ae != a_end) && (ae != b_end))
         return force;
      Vector2D springVector = getVector();
      double stretch = springVector.module() - largonat;
      force = springVector.unitary().times(stretch*rigidez);
      if (ae == a_end) return force;
      return force.times(-1.0);
   }
   
}
 
