
public class Vector2D implements Cloneable {

    private double x, y;  // we will use cartesian coordinates
/** 
 * Constructor de Vector .  Inicializa las coordendas x y del vector a 0
 
 */ 
    public Vector2D() {
        x = y = 0;
    }
/** 
 * Constructor Vector2D.   Inicializa las coordendas
 * @param x coordenada x
 * @param y coordenada y vertical
 */  
    public Vector2D(double x, double y) {
        this.x = x;
        this.y = y;
    }
/** 
 * Retorna la coordenada X del vector
 */  
    public double getX() {
        return x;
    }
/** 
 * Retorna la coordenada y del vector
 */  
    public double getY() {
        return y;
    }
/** 
 *  Cambia las coordenadas del vector
 * @param x coordenada x
 * @param y coordenada y vertical
 */  
    public void set(double x, double y) { // to set a new position
        this.x = x;
        this.y = y;
    }
/** 
 *  Implementación de la suma del vector
 * 
 * @param v Vector que se sumará al que llame al metodo plus
 */
    public Vector2D plus(Vector2D v) {
        try {
            if (v == null) {
                return (Vector2D) clone();
            }
        } catch (CloneNotSupportedException e) {
            return null;
        }
        return new Vector2D(x + v.x, y + v.y);
    }
/** 
 *  Implementación de multiplicacion por un escalar a un vector
 * 
 * @param escalar valor por el que se multiplicara
 */
    public Vector2D times(double scalar) {
        return new Vector2D(x * scalar, y * scalar);
    }
/** 
 *  Implementación de multiplicacion de coordenadas del vector
 * 
 * @param m1 multiplicacion de la coordenada x del vector
 * @param m2 multiplicacion de la coordenada y del vector
 */
    public Vector2D mult(double m1, double m2) {
        return new Vector2D(x * m1, y * m2);
    }
/** 
 *  Implementación de resta de vectores
 * 
 * @param v Vector que se restara al que llame al metodo plus
 */
    public Vector2D minus(Vector2D v) {
        try {
            if (v == null) {
                return (Vector2D) clone();
            }
        } catch (CloneNotSupportedException e) {
            return null;
        }
        return new Vector2D(x - v.x, y - v.y);
    }

    public double module() {
        return Math.sqrt(x * x + y * y);
    }

    public Vector2D unitary() {
        return times(1 / module());
    }

    public static String getDescription() {
        return "(x,y)";
    }

    public String toString() {
        return x + "," + y;
    }
}
